@@ -17,7 +17,7 @@
 #include <generated_executor.h>
 #include <CBindingCache.h>
 #include <common.h>
-
+#include <sys/time.h>
 int get_compss_worker_lock() {
     return 0;
 }
@@ -29,7 +29,12 @@ int release_compss_worker_lock() {
 int main(int argc, char **argv) {
     init_env_vars();
     CBindingCache *cache = new CBindingCache();
+    struct timeval t_comp_start, t_comp_end;
+   gettimeofday(&t_comp_start, NULL);
     int out = execute(argc, argv, cache, 1);
+    gettimeofday(&t_comp_end, NULL);
+   double total_msecs = (((t_comp_end.tv_sec - t_comp_start.tv_sec) * 1000000) + (t_comp_end.tv_usec - t_comp_start.tv_usec))/1000;
+    printf("COMPSs task executor time: %f ms\n",total_msecs);
     if (out == 0) {
         printf("Task executed successfully");
     } else {
