@@ -147,6 +147,53 @@ void generate_worker_prolog() {
     fprintf(workerFile, "using namespace std;\n");
     fprintf(workerFile, "\n");
 }
+
+/*
+ * Generate prototype for the initThread method at worker
+ */
+void generate_initThread_prototype(FILE *outFile) {
+	fprintf(outFile, "void GE_initThread() {\n");
+    fprintf(outFile, "\n");
+}
+
+void generate_worker_initThread() {
+	generate_initThread_prototype(workerFile);
+
+    fprintf(workerFile, "#ifdef OMPSS_ENABLED\n");
+    fprintf(workerFile, "\t\t nanos_admit_current_thread();\n");
+    fprintf(workerFile, "#endif\n");
+    fprintf(workerFile, "}\n");
+
+}
+
+void generate_finishThread_prototype(FILE *outFile) {
+	fprintf(outFile, "void GE_finishThread() {\n");
+    fprintf(outFile, "\n");
+}
+
+void generate_worker_finishThread() {
+	generate_finishThread_prototype(workerFile);
+
+    fprintf(workerFile, "#ifdef OMPSS_ENABLED\n");
+    fprintf(workerFile, "\t\t nanos_leave_team();\n");
+    fprintf(workerFile, "\t\t nanos_expel_current_thread();\n");
+    fprintf(workerFile, "#endif\n");
+    fprintf(workerFile, "}\n");
+
+}
+
+void generate_initThread_empty(FILE *outFile) {
+	generate_initThread_prototype(outFile);
+	fprintf(outFile, "return;\n");
+	fprintf(outFile, "}\n");
+}
+
+void generate_finishThread_empty(FILE *outFile) {
+	generate_finishThread_prototype(outFile);
+	fprintf(outFile, "return;\n");
+	fprintf(outFile, "}\n");
+}
+
 /*
  * Generate prototype for the execute method at worker
  */
@@ -634,15 +681,25 @@ static void generate_remove_func(FILE *outFile, Types current_types) {
         // Remove treatment
         char * dataType = current_types.types[i].name;
         fprintf(outFile, "\t\t case %d:\n", i);
-        fprintf(outFile, "\t\t\t %s *data_%d;\n", dataType, i);
-        fprintf(outFile, "\t\t\t data_%d = (%s*)cp.pointer;\n", i, dataType);
-        fprintf(outFile, "\t\t\t if(is_debug()){\n"); //Open if debug
-        fprintf(outFile, "\t\t\t cout << \"[C Binding] Deleting object from memory...\" << endl;\n");
-        fprintf(outFile, "\t\t\t }");// end if debug;
-        fprintf(outFile, "\t\t\t delete(data_%d);\n",i);
-        fprintf(outFile, "\t\t\t if(is_debug()){\n"); //Open if debug
-        fprintf(outFile, "\t\t\t\t cout << \"[C Binding] Object has been removed.\" << endl;\n");
-        fprintf(outFile, "\t\t\t }");// end if debug;
+        fprintf(outFile, "\t\t\t if (cp.pointer != NULL) {\n ");
+        //fprintf(outFile, "\t\t\t\t %s *data_%d = (%s*)cp.pointer;\n", dataType, i, dataType);
+        fprintf(outFile, "\t\t\t\t if(is_debug()){\n"); //Open if debug
+        fprintf(outFile, "\t\t\t\t\t cout << \"[C Binding] Deleting object from memory \" << hex << cp.pointer << endl;\n");
+        fprintf(outFile, "\t\t\t\t }\n");// end if debug;
+        /*fprintf(outFile, "\t\t\t\t if (data_%d != NULL){\n",i);
+        fprintf(outFile, "\t\t\t\t\t delete(data_%d);\n",i);
+        fprintf(outFile, "\t\t\t\t }\n");*/
+        fprintf(outFile, "\t\t\t\t if (cp.elements > 0 ){\n");
+        fprintf(outFile, "\t\t\t\t\t cout << \"[C Binding] Deleting array\" << endl;\n");
+        fprintf(outFile, "\t\t\t\t\t free(cp.pointer);\n");
+        fprintf(outFile, "\t\t\t\t } else {\n");
+        fprintf(outFile, "\t\t\t\t\t cout << \"[C Binding] Deleting object\" << endl;\n");
+        fprintf(outFile, "\t\t\t\t\t delete((%s*) cp.pointer);\n", dataType);
+        fprintf(outFile, "\t\t\t\t }\n");// end if cp.elements
+        fprintf(outFile, "\t\t\t\t if(is_debug()){\n"); //Open if debug
+        fprintf(outFile, "\t\t\t\t\t cout << \"[C Binding] Object has been removed.\" << endl;\n");
+        fprintf(outFile, "\t\t\t\t }\n");// end if debug;
+        fprintf(outFile, "\t\t\t }\n");
         fprintf(outFile, "\t\t\t break;\n");
     }
     fprintf(outFile, "\t\t default:;\n");
@@ -885,7 +942,8 @@ static void generate_class_includes_and_check_types(FILE *outFile, Types *curren
  * Generate buffer to add task parameters before calling the task execution
  */
 static void generate_parameter_buffers(FILE *outFile, function *func) {
-    int k = 0;
+	fprintf(outFile, "\t debug_printf(\"Detected task %s\\n\");\n", func->name);
+	int k = 0;
     //There is a target object
     if (( func->classname != NULL ) && (func->access_static == 0)) k = k + 5;
     //There is a return type
@@ -1075,6 +1133,7 @@ static void generate_execute_task_call(FILE *outFile, function *func) {
     char *hasTarget = strdup("false");
 
     int arg_count = func->argument_count;
+    int num_returns = 0;
 
     if ( func->classname != NULL ) {
         class_name = func->classname;
@@ -1086,10 +1145,11 @@ static void generate_execute_task_call(FILE *outFile, function *func) {
 
     if ( func->return_type != void_dt ) {
         arg_count++;
+	num_returns = 1;
     }
 
     fprintf(outFile, "\t char *method_name = strdup(\"%s\");\n", func->name);
-    fprintf(outFile, "\t GS_ExecuteTask(0L, \"%s\", method_name, 0, %s, %d, (void**)arrayObjs);\n", class_name, hasTarget, arg_count);
+    fprintf(outFile, "\t GS_ExecuteTask(0L, \"%s\", method_name, 0, %s, %d, %d, (void**)arrayObjs);\n", class_name, hasTarget, num_returns, arg_count);
     fprintf(outFile, "\t debug_printf(\"[   BINDING]  -  @%%s  -  Task submited in the runtime\\n\", method_name);\n");
     fprintf(outFile, "\n");
 
@@ -1298,33 +1358,19 @@ static void add_object_or_array_arg_worker_treatment(FILE *outFile, argument *ar
     fprintf(outFile, "\t\t\t\t cout << \"[C Binding] Data ID for parameter %s not found in argument value. Reseting filename to \" << %s_filename_og << endl << flush;\n", arg->name, arg->name);
     fprintf(outFile, "\t\t\t\t %s_filename = %s_filename_og ;\n", arg->name, arg->name);
     fprintf(outFile, "\t\t\t\t %s_dest_id = %s_filename_og ;\n", arg->name, arg->name);
-    fprintf(outFile, "\t\t\t\t compss_pointer %s_cp;\n", arg->name);
-    fprintf(outFile, "\t\t\t\t %s_cp.type= %d;\n", arg->name, position);
-    fprintf(outFile, "\t\t\t\t %s_cp.elements= %s_elements;\n", arg->name, arg->name);
-    fprintf(outFile, "\t\t\t\t %s_cp.pointer = (void*)%s;\n", arg->name, arg->name);
-    fprintf(outFile, "\t\t\t\t %s_cp.size = %s_cp.elements*sizeof(%s);\n", arg->name, arg->name, arg->classname);
-    fprintf(outFile, "\t\t\t\t int res = cache->deserializeFromFile(%s_filename, %s_cp);\n", arg->name, arg->name);
-    fprintf(outFile, "\t\t\t\t if (res != 0){\n");
-    fprintf(outFile, "\t\t\t\t\t cout << \"[C Binding] Error deserializing object %s from file \"<< %s_filename << endl << flush;\n", arg->name, arg->name);
-    fprintf(outFile, "\t\t\t\t\t return res;\n");
-    fprintf(outFile, "\t\t\t\t }\n");
-    fprintf(outFile, "\t\t\t\t %s = (%s*)%s_cp.pointer;\n", arg->name, arg->classname, arg->name);
-
-    /*switch (arg->dir){
-    case in_dir:
-    case inout_dir:
-    {	//char *elems_suff = "_elements";
-        //char *file_suff = "_filename";
-        char *elements = concat(arg->name, "_elements");
-        char *filename = concat(arg->name, "_filename");
-
-        //add_deserialization(outFile, arg->type, arg->name, filename, elements, "\t\t\t\t");
-        free(filename);
-        free(elements);
-        break;
+    if (arg->dir != out_dir){
+    	fprintf(outFile, "\t\t\t\t compss_pointer %s_cp;\n", arg->name);
+    	fprintf(outFile, "\t\t\t\t %s_cp.type= %d;\n", arg->name, position);
+    	fprintf(outFile, "\t\t\t\t %s_cp.elements= %s_elements;\n", arg->name, arg->name);
+    	fprintf(outFile, "\t\t\t\t %s_cp.pointer = (void*)%s;\n", arg->name, arg->name);
+    	fprintf(outFile, "\t\t\t\t %s_cp.size = %s_cp.elements*sizeof(%s);\n", arg->name, arg->name, arg->classname);
+    	fprintf(outFile, "\t\t\t\t int res = cache->deserializeFromFile(%s_filename, %s_cp);\n", arg->name, arg->name);
+    	fprintf(outFile, "\t\t\t\t if (res != 0){\n");
+    	fprintf(outFile, "\t\t\t\t\t cout << \"[C Binding] Error deserializing object %s from file \"<< %s_filename << endl << flush;\n", arg->name, arg->name);
+    	fprintf(outFile, "\t\t\t\t\t return res;\n");
+    	fprintf(outFile, "\t\t\t\t }\n");
+    	fprintf(outFile, "\t\t\t\t %s = (%s*)%s_cp.pointer;\n", arg->name, arg->classname, arg->name);
     }
-    default:;
-    }*/
     fprintf(outFile, "\t\t\t } //end obtain object as file \n");
     fprintf(outFile, "\t\t\t arg_offset += 1;\n\n");
 }
@@ -1838,6 +1884,8 @@ void generate_body(void) {
     generate_executor_end();
     printf("\t Generating remove and serialize functions... \n");
     fflush(NULL);
+    generate_initThread_empty(stubsFile);
+    generate_finishThread_empty(stubsFile);
     generate_execute_empty(stubsFile);
     generate_remove_func(workerFile, current_types);
     generate_remove_func(stubsFile, current_types);
@@ -1851,4 +1899,6 @@ void generate_body(void) {
     generate_deserializeFromStream_func(stubsFile, current_types);
     generate_deserializeFromFile_func(workerFile, current_types);
     generate_deserializeFromFile_func(stubsFile, current_types);
+    generate_worker_initThread();
+    generate_worker_finishThread();
 }
