@@ -1,5 +1,6 @@
+#!/usr/bin/python
 #
-#  Copyright Barcelona Supercomputing Center (www.bsc.es)
+#  Copyright 2002-2018 Barcelona Supercomputing Center (www.bsc.es)
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,7 +13,9 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
-#
+# 
+
+# -*- coding: utf-8 -*-
 
 """
 PyCOMPSs API - BINARY
@@ -20,40 +23,64 @@ PyCOMPSs API - BINARY
     This file contains the class constraint, needed for the binary task
     definition through the decorator.
 """
+
 import inspect
 import logging
 import os
 from functools import wraps
-from pycompss.runtime.binding import register_ce
-from pycompss.util.location import i_am_at_master
+from pycompss.util.location import at_master
+from pycompss.util.location import within_scope
+
+if __debug__:
+    logger = logging.getLogger(__name__)
 
 
-logger = logging.getLogger(__name__)
-
-
-class binary(object):
+class Binary(object):
     """
     This decorator also preserves the argspec, but includes the __init__ and
     __call__ methods, useful on mpi task creation.
     """
+
     def __init__(self, *args, **kwargs):
-        # store arguments passed to the decorator
-        self.args = args
-        self.kwargs = kwargs
-        logger.debug("Init @binary decorator...")
+        """
+        Store arguments passed to the decorator
         # self = itself.
         # args = not used.
         # kwargs = dictionary with the given constraints.
 
+        :param args: Arguments
+        :param kwargs: Keyword arguments
+        """
+
+        self.args = args
+        self.kwargs = kwargs
+        self.scope = within_scope()
+        if self.scope and __debug__:
+            logger.debug("Init @binary decorator...")
+
     def __call__(self, func):
+        """
+        Parse and set the binary parameters within the task core element.
 
-        if i_am_at_master():
+        :param func: Function to decorate
+        :return: Decorated function.
+        """
+
+        if not self.scope:
+            # from pycompss.api.dummy.binary import binary as dummy_binary
+            # d_b = dummy_binary(self.args, self.kwargs)
+            # return d_b.__call__(func)
+            raise Exception("The binary decorator only works within PyCOMPSs framework.")
+
+        if at_master():
             # master code
-            mod = inspect.getmodule(func)
-            self.module = mod.__name__    # not func.__module__
+            from pycompss.runtime.binding import register_ce
 
-            if(self.module == '__main__' or
-               self.module == 'pycompss.runtime.launch'):
+            mod = inspect.getmodule(func)
+            self.module = mod.__name__  # not func.__module__
+
+            if (self.module == '__main__' or
+                    self.module == 'pycompss.runtime.launch'):
                 # The module where the function is defined was run as __main__,
                 # we need to find out the real module name.
 
@@ -80,26 +107,27 @@ class binary(object):
                         break
                 self.module = mod_name
 
-            # Include the registering info related to @Binary
+            # Include the registering info related to @binary
 
-            # Retrieve the base coreElement established at @task decorator
-            coreElement = func.__to_register__
+            # Retrieve the base core_element established at @task decorator
+            core_element = func.__to_register__
             # Update the core element information with the mpi information
-            coreElement.set_implType("BINARY")
-            binary = self.kwargs['binary']
+            core_element.set_impl_type("BINARY")
+            _binary = self.kwargs['binary']
             if 'workingDir' in self.kwargs:
-                workingDir = self.kwargs['workingDir']
+                working_dir = self.kwargs['workingDir']
             else:
-                workingDir = '[unassigned]'   # Empty or '[unassigned]'
-            implSignature = 'BINARY.' + binary
-            coreElement.set_implSignature(implSignature)
-            implArgs = [binary, workingDir]
-            coreElement.set_implTypeArgs(implArgs)
-            func.__to_register__ = coreElement
+                working_dir = '[unassigned]'   # Empty or '[unassigned]'
+            impl_signature = 'BINARY.' + _binary
+            core_element.set_impl_signature(impl_signature)
+            impl_args = [_binary, working_dir]
+            core_element.set_impl_type_args(impl_args)
+            func.__to_register__ = core_element
             # Do the task register if I am the top decorator
             if func.__who_registers__ == __name__:
-                logger.debug("[@BINARY] I have to do the register of function %s in module %s" % (func.__name__, self.module))
-                register_ce(coreElement)
+                if __debug__:
+                    logger.debug("[@BINARY] I have to do the register of function %s in module %s" % (func.__name__, self.module))
+                register_ce(core_element)
         else:
             # worker code
             pass
@@ -107,7 +135,8 @@ class binary(object):
         @wraps(func)
         def binary_f(*args, **kwargs):
             # This is executed only when called.
-            logger.debug("Executing binary_f wrapper.")
+            if __debug__:
+                logger.debug("Executing binary_f wrapper.")
 
             if len(args) > 0:
                 # The 'self' for a method function is passed as args[0]
@@ -129,5 +158,13 @@ class binary(object):
                     setattr(slf, k, v)
 
             return ret
+
         binary_f.__doc__ = func.__doc__
         return binary_f
+
+
+# ############################################################################# #
+# #################### BINARY DECORATOR ALTERNATIVE NAME ###################### #
+# ############################################################################# #
+
+binary = Binary
