@@ -28,14 +28,14 @@ import inspect
 import logging
 import os
 from functools import wraps
-from pycompss.util.location import i_am_at_master
-from pycompss.util.location import i_am_within_scope
+from pycompss.util.location import at_master
+from pycompss.util.location import within_scope
 
 if __debug__:
     logger = logging.getLogger(__name__)
 
 
-class decaf(object):
+class Decaf(object):
     """
     This decorator also preserves the argspec, but includes the __init__ and
     __call__ methods, useful on mpi task creation.
@@ -54,7 +54,7 @@ class decaf(object):
 
         self.args = args
         self.kwargs = kwargs
-        self.scope = i_am_within_scope()
+        self.scope = within_scope()
         if self.scope:
             if __debug__:
                 logger.debug("Init @decaf decorator...")
@@ -93,7 +93,7 @@ class decaf(object):
             # return d_d.__call__(func)
             raise Exception("The decaf decorator only works within PyCOMPSs framework.")
 
-        if i_am_at_master():
+        if at_master():
             # master code
             from pycompss.runtime.binding import register_ce
 
@@ -197,3 +197,10 @@ class decaf(object):
 
         decaf_f.__doc__ = func.__doc__
         return decaf_f
+
+
+# ############################################################################# #
+# ##################### DECAF DECORATOR ALTERNATIVE NAME ###################### #
+# ############################################################################# #
+
+decaf = Decaf
