@@ -28,14 +28,14 @@ import inspect
 import logging
 import os
 from functools import wraps
-from pycompss.util.location import i_am_at_master
-from pycompss.util.location import i_am_within_scope
+from pycompss.util.location import at_master
+from pycompss.util.location import within_scope
 
 if __debug__:
     logger = logging.getLogger(__name__)
 
 
-class mpi(object):
+class Mpi(object):
     """
     This decorator also preserves the argspec, but includes the __init__ and
     __call__ methods, useful on mpi task creation.
@@ -54,7 +54,7 @@ class mpi(object):
 
         self.args = args
         self.kwargs = kwargs
-        self.scope = i_am_within_scope()
+        self.scope = within_scope()
         if self.scope:
             if __debug__:
                 logger.debug("Init @mpi decorator...")
@@ -93,7 +93,7 @@ class mpi(object):
             # return d_m.__call__(func)
             raise Exception("The mpi decorator only works within PyCOMPSs framework.")
 
-        if i_am_at_master():
+        if at_master():
             # master code
             from pycompss.runtime.binding import register_ce
 
@@ -187,3 +187,10 @@ class mpi(object):
 
         mpi_f.__doc__ = func.__doc__
         return mpi_f
+
+
+# ############################################################################# #
+# ###################### MPI DECORATOR ALTERNATIVE NAME ####################### #
+# ############################################################################# #
+
+mpi = Mpi
