@@ -1,5 +1,6 @@
+#!/usr/bin/python
 #
-#  Copyright Barcelona Supercomputing Center (www.bsc.es)
+#  Copyright 2002-2018 Barcelona Supercomputing Center (www.bsc.es)
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,7 +13,9 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
-#
+# 
+
+# -*- coding: utf-8 -*-
 
 """
 PyCOMPSs API - OMPSS
@@ -20,49 +23,85 @@ PyCOMPSs API - OMPSS
     This file contains the class constraint, needed for the OmpSs task
     definition through the decorator.
 """
+
 import inspect
 import logging
 import os
 from functools import wraps
-from pycompss.runtime.binding import register_ce
-from pycompss.util.location import i_am_at_master
+from pycompss.util.location import at_master
+from pycompss.util.location import within_scope
+
+if __debug__:
+    logger = logging.getLogger(__name__)
 
 
-logger = logging.getLogger(__name__)
-
-
-class ompss(object):
+class Ompss(object):
     """
     This decorator also preserves the argspec, but includes the __init__ and
     __call__ methods, useful on mpi task creation.
     """
+
     def __init__(self, *args, **kwargs):
-        # store arguments passed to the decorator
-        self.args = args
-        self.kwargs = kwargs
-        logger.debug("Init @ompss decorator...")
-
-        # Get the computing nodes: This parameter will have to go down until
-        # execution when invoked.
-        if 'computingNodes' not in self.kwargs:
-            self.kwargs['computingNodes'] = 1
-        else:
-            self.kwargs['computingNodes'] = kwargs['computingNodes']
-        logger.debug("This MPI task will have " + str(self.kwargs['computingNodes']) + " computing nodes.")
-
+        """
+        Store arguments passed to the decorator
         # self = itself.
         # args = not used.
-        # kwargs = dictionary with the given constraints.
+        # kwargs = dictionary with the given ompss parameters
+
+        :param args: Arguments
+        :param kwargs: Keyword arguments
+        """
+
+        self.args = args
+        self.kwargs = kwargs
+        self.scope = within_scope()
+        if self.scope:
+            if __debug__:
+                logger.debug("Init @ompss decorator...")
+
+            # Get the computing nodes: This parameter will have to go down until
+            # execution when invoked.
+            if 'computingNodes' not in self.kwargs:
+                self.kwargs['computingNodes'] = 1
+            else:
+                computing_nodes = kwargs['computingNodes']
+                if isinstance(computing_nodes, int):
+                    self.kwargs['computingNodes'] = kwargs['computingNodes']
+                elif isinstance(computing_nodes, str) and computing_nodes.strip().startswith('$'):
+                    env_var = computing_nodes.strip()[1:]  # Remove $
+                    if env_var.startswith('{'):
+                        env_var = env_var[1:-1]  # remove brackets
+                    self.kwargs['computingNodes'] = int(os.environ[env_var])
+                else:
+                    raise Exception("Wrong Computing Nodes value at MPI decorator.")
+            if __debug__:
+                logger.debug("This OMPSs task will have " + str(self.kwargs['computingNodes']) + " computing nodes.")
+        else:
+            pass
 
     def __call__(self, func):
+        """
+        Parse and set the ompss parameters within the task core element.
 
-        if i_am_at_master():
+        :param func: Function to decorate
+        :return: Decorated function.
+        """
+
+        if not self.scope:
+            # from pycompss.api.dummy.ompss import ompss as dummy_ompss
+            # d_o = dummy_ompss(self.args, self.kwargs)
+            # return d_o.__call__(func)
+            raise Exception("The ompss decorator only works within PyCOMPSs framework.")
+
+        if at_master():
             # master code
-            mod = inspect.getmodule(func)
-            self.module = mod.__name__    # not func.__module__
+            from pycompss.runtime.binding import register_ce
 
-            if(self.module == '__main__' or
-               self.module == 'pycompss.runtime.launch'):
+            mod = inspect.getmodule(func)
+            self.module = mod.__name__  # not func.__module__
+
+            if (self.module == '__main__' or
+                    self.module == 'pycompss.runtime.launch'):
                 # The module where the function is defined was run as __main__,
                 # we need to find out the real module name.
 
@@ -89,26 +128,27 @@ class ompss(object):
                         break
                 self.module = mod_name
 
-            # Include the registering info related to @OmpSs
+            # Include the registering info related to @ompss
 
-            # Retrieve the base coreElement established at @task decorator
-            coreElement = func.__to_register__
+            # Retrieve the base core_element established at @task decorator
+            core_element = func.__to_register__
             # Update the core element information with the mpi information
-            coreElement.set_implType("OMPSS")
+            core_element.set_impl_type("OMPSS")
             binary = self.kwargs['binary']
             if 'workingDir' in self.kwargs:
-                workingDir = self.kwargs['workingDir']
+                working_dir = self.kwargs['workingDir']
             else:
-                workingDir = '[unassigned]'   # Empty or '[unassigned]'
-            implSignature = 'OMPSS.' + binary
-            coreElement.set_implSignature(implSignature)
-            implArgs = [binary, workingDir]
-            coreElement.set_implTypeArgs(implArgs)
-            func.__to_register__ = coreElement
+                working_dir = '[unassigned]'   # Empty or '[unassigned]'
+            impl_signature = 'OMPSS.' + binary
+            core_element.set_impl_signature(impl_signature)
+            impl_args = [binary, working_dir]
+            core_element.set_impl_type_args(impl_args)
+            func.__to_register__ = core_element
             # Do the task register if I am the top decorator
             if func.__who_registers__ == __name__:
-                logger.debug("[@OMPSS] I have to do the register of function %s in module %s" % (func.__name__, self.module))
-                register_ce(coreElement)
+                if __debug__:
+                    logger.debug("[@OMPSS] I have to do the register of function %s in module %s" % (func.__name__, self.module))
+                register_ce(core_element)
         else:
             # worker code
             pass
@@ -116,7 +156,8 @@ class ompss(object):
         @wraps(func)
         def ompss_f(*args, **kwargs):
             # This is executed only when called.
-            logger.debug("Executing ompss_f wrapper.")
+            if __debug__:
+                logger.debug("Executing ompss_f wrapper.")
 
             # Set the computingNodes variable in kwargs for its usage
             # in @task decorator
@@ -142,5 +183,13 @@ class ompss(object):
                     setattr(slf, k, v)
 
             return ret
+
         ompss_f.__doc__ = func.__doc__
         return ompss_f
+
+
+# ############################################################################# #
+# ##################### OMPSs DECORATOR ALTERNATIVE NAME ###################### #
+# ############################################################################# #
+
+ompss = Ompss
