@@ -24,7 +24,7 @@
 
   # Application arguments
   NSIZE=8
-  TSIZE=1
+  TSIZE=2
 
   export ComputingUnits=1
 
@@ -44,7 +44,7 @@
   if [ -f "${LOG_DIR}/monitor/complete_graph.dot" ]; then
     cp "${LOG_DIR}"/monitor/complete_graph.dot "${RESULTS_DIR}"
     gengraph "${RESULTS_DIR}"/complete_graph.dot
-    dot -Tpng "${RESULTS_DIR}"/complete_graph.dot > "${RESULTS_DIR}"/complete_graph.png
+    dot -Tpng -Gnewrank=true "${RESULTS_DIR}"/complete_graph.dot > "${RESULTS_DIR}"/complete_graph.png
   fi
   if [ -d "${LOG_DIR}/trace/" ]; then
     mkdir -p "${RESULTS_DIR}"/trace
