@@ -1,6 +1,7 @@
 #!/bin/bash
 
-export EXTRAE_HOME=@sub_PREFIXDIR@
+source @sub_PREFIXDIR@/etc/extrae.sh
+
 export EXTRAE_CONFIG_FILE=../extrae.xml
 #export LD_PRELOAD=${EXTRAE_HOME}/lib/libmpitrace.so # For C apps
 export LD_PRELOAD=${EXTRAE_HOME}/lib/libmpitracef.so # For Fortran apps
