@@ -1,8 +1,8 @@
 #!/bin/bash
 
+source @sub_PREFIXDIR@/etc/extrae.sh
+
 export EXTRAE_CONFIG_FILE=extrae.xml
-export EXTRAE_HOME=@sub_PREFIXDIR@
-source ${EXTRAE_HOME}/etc/extrae.sh
 export EXECUTABLE=./pi_instrumentedf # This is taken by extrae.xml
 
 ${EXECUTABLE}
