@@ -23,22 +23,6 @@
 
 #include "common.h"
 
-#if defined(MPI3)
-#define MPI3_CONST const
-#define MPI3_VOID_P_CAST (void *)
-#define MPI3_CHAR_P_CAST (char *)
-#define MPI3_F_INT_P_CAST (MPI_Fint *)
-#define MPI3_C_INT_P_CAST (int *)
-#define MPI3_MPI_INFO_P_CAST (MPI_Info *)
-#else
-#define MPI3_CONST
-#define MPI3_VOID_P_CAST
-#define MPI3_CHAR_P_CAST
-#define MPI3_F_INT_P_CAST
-#define MPI3_C_INT_P_CAST
-#define MPI3_MPI_INFO_P_CAST
-#endif
-
 #ifdef HAVE_STDIO_H
 # include <stdio.h>
 #endif
@@ -58,17 +42,8 @@
 #include "wrapper.h"
 #include "mpi_wrapper.h"
 #include "mpi_interface_coll_helper.h"
-
-#if defined(ENABLE_LOAD_BALANCING)
-# if defined(FORTRAN_SYMBOLS)
-#  include "MPI_interfaceF.h"
-# endif
-# if defined(C_SYMBOLS)
-#  include "MPI_interface.h"
-# endif
-#endif
-
-#include "mpi_interface_coll_helper.h"
+#include "mpi_interface.h"
+#include "dlb.h"
 
 #if defined(C_SYMBOLS) && defined(FORTRAN_SYMBOLS)
 # define COMBINED_SYMBOLS
@@ -115,16 +90,14 @@ int NAME_ROUTINE_C(MPI_Reduce) (MPI3_CONST void *sendbuf, void *recvbuf, int cou
 {
 	int res;
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Reduce_enter (MPI3_VOID_P_CAST sendbuf, recvbuf, count, datatype, op, root, comm);
-#endif
+	DLB(DLB_MPI_Reduce_enter, MPI3_VOID_P_CAST sendbuf, recvbuf, count, datatype, op, root, comm);
 
 	Extrae_MPI_ProcessCollectiveCommunicator (comm);
 
 	if (mpitrace_on)
 	{
 		DEBUG_INTERFACE(ENTER)
-		Backend_Enter_Instrumentation (2+Caller_Count[CALLER_MPI]);
+		Backend_Enter_Instrumentation ();
 		res = MPI_Reduce_C_Wrapper 
 			(MPI3_VOID_P_CAST sendbuf, recvbuf, count, datatype, op, root, comm);
 		Backend_Leave_Instrumentation ();
@@ -133,9 +106,7 @@ int NAME_ROUTINE_C(MPI_Reduce) (MPI3_CONST void *sendbuf, void *recvbuf, int cou
 	else
 		res = PMPI_Reduce (sendbuf, recvbuf, count, datatype, op, root, comm);
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Reduce_leave ();
-#endif
+	DLB(DLB_MPI_Reduce_leave);
 
 	return res;
 }
@@ -148,16 +119,14 @@ int NAME_ROUTINE_C(MPI_Reduce_scatter) (MPI3_CONST void *sendbuf, void *recvbuf,
 {
 	int res;
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Reduce_scatter_enter (MPI3_VOID_P_CAST sendbuf, recvbuf, MPI3_C_INT_P_CAST recvcounts, datatype, op, comm);
-#endif
+	DLB(DLB_MPI_Reduce_scatter_enter, MPI3_VOID_P_CAST sendbuf, recvbuf, MPI3_C_INT_P_CAST recvcounts, datatype, op, comm);
 
 	Extrae_MPI_ProcessCollectiveCommunicator (comm);
 
 	if (mpitrace_on)
 	{
 		DEBUG_INTERFACE(ENTER)
-		Backend_Enter_Instrumentation (2+Caller_Count[CALLER_MPI]);
+		Backend_Enter_Instrumentation ();
 		res = MPI_Reduce_Scatter_C_Wrapper (MPI3_VOID_P_CAST sendbuf, recvbuf, MPI3_C_INT_P_CAST recvcounts, datatype,
 			op, comm);
 		Backend_Leave_Instrumentation ();
@@ -167,9 +136,7 @@ int NAME_ROUTINE_C(MPI_Reduce_scatter) (MPI3_CONST void *sendbuf, void *recvbuf,
 		res = PMPI_Reduce_scatter (sendbuf, recvbuf, recvcounts, datatype, op,
 			comm);
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Reduce_scatter_leave ();
-#endif
+	DLB(DLB_MPI_Reduce_scatter_leave);
 
 	return res;
 }
@@ -182,16 +149,14 @@ int NAME_ROUTINE_C(MPI_Allreduce) (MPI3_CONST void *sendbuf, void *recvbuf, int
 {
 	int res;
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Allreduce_enter (MPI3_VOID_P_CAST sendbuf, recvbuf, count, datatype, op, comm);
-#endif
+	DLB(DLB_MPI_Allreduce_enter, MPI3_VOID_P_CAST sendbuf, recvbuf, count, datatype, op, comm);
 
 	Extrae_MPI_ProcessCollectiveCommunicator (comm);
 
 	if (mpitrace_on)
 	{
 		DEBUG_INTERFACE(ENTER)
-		Backend_Enter_Instrumentation (2+Caller_Count[CALLER_MPI]);
+		Backend_Enter_Instrumentation ();
 		res = MPI_Allreduce_C_Wrapper (MPI3_VOID_P_CAST sendbuf, recvbuf, count, datatype, op, comm);
 		Backend_Leave_Instrumentation ();
 		DEBUG_INTERFACE(LEAVE)
@@ -199,9 +164,7 @@ int NAME_ROUTINE_C(MPI_Allreduce) (MPI3_CONST void *sendbuf, void *recvbuf, int
 	else
 		res = PMPI_Allreduce (sendbuf, recvbuf, count, datatype, op, comm);
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Allreduce_leave ();
-#endif
+	DLB(DLB_MPI_Allreduce_leave);
 
 	return res;
 }
@@ -213,16 +176,14 @@ int NAME_ROUTINE_C(MPI_Barrier) (MPI_Comm comm)
 {
 	int res;
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Barrier_enter (comm);
-#endif
+	DLB(DLB_MPI_Barrier_enter, comm);
 
 	Extrae_MPI_ProcessCollectiveCommunicator (comm);
 
 	if (mpitrace_on)
 	{
 		DEBUG_INTERFACE(ENTER)
-		Backend_Enter_Instrumentation (2+Caller_Count[CALLER_MPI]);
+		Backend_Enter_Instrumentation ();
 		res =  MPI_Barrier_C_Wrapper (comm);
 		Backend_Leave_Instrumentation ();
 		DEBUG_INTERFACE(LEAVE)
@@ -230,9 +191,7 @@ int NAME_ROUTINE_C(MPI_Barrier) (MPI_Comm comm)
 	else
 		res = PMPI_Barrier (comm);
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Barrier_leave ();
-#endif
+	DLB(DLB_MPI_Barrier_leave);
 
 	return res;
 }
@@ -245,16 +204,14 @@ int NAME_ROUTINE_C(MPI_Bcast) (void *buffer, int count, MPI_Datatype datatype,
 {
 	int res;
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Bcast_enter (buffer, count, datatype, root, comm);
-#endif
+	DLB(DLB_MPI_Bcast_enter, buffer, count, datatype, root, comm);
 
 	Extrae_MPI_ProcessCollectiveCommunicator (comm);
 
 	if (mpitrace_on)
 	{
 		DEBUG_INTERFACE(ENTER)
-		Backend_Enter_Instrumentation (2+Caller_Count[CALLER_MPI]);
+		Backend_Enter_Instrumentation ();
 		res = MPI_BCast_C_Wrapper (buffer, count, datatype, root, comm);
 		Backend_Leave_Instrumentation ();
 		DEBUG_INTERFACE(LEAVE)
@@ -262,9 +219,7 @@ int NAME_ROUTINE_C(MPI_Bcast) (void *buffer, int count, MPI_Datatype datatype,
 	else
 		res = PMPI_Bcast (buffer, count, datatype, root, comm);
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Bcast_leave ();
-#endif
+	DLB(DLB_MPI_Bcast_leave);
 
 	return res;
 }
@@ -278,16 +233,14 @@ int NAME_ROUTINE_C(MPI_Alltoall) (MPI3_CONST void *sendbuf, int sendcount,
 {
 	int res;
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Alltoall_enter (MPI3_VOID_P_CAST sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
-#endif
+	DLB(DLB_MPI_Alltoall_enter, MPI3_VOID_P_CAST sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
 
 	Extrae_MPI_ProcessCollectiveCommunicator (comm);
 
 	if (mpitrace_on)
 	{
 		DEBUG_INTERFACE(ENTER)
-		Backend_Enter_Instrumentation (2+Caller_Count[CALLER_MPI]);
+		Backend_Enter_Instrumentation ();
 		res = MPI_Alltoall_C_Wrapper
 		  (MPI3_VOID_P_CAST sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
 		Backend_Leave_Instrumentation ();
@@ -297,9 +250,7 @@ int NAME_ROUTINE_C(MPI_Alltoall) (MPI3_CONST void *sendbuf, int sendcount,
 		res = PMPI_Alltoall
 		  (sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Alltoall_leave ();
-#endif
+	DLB(DLB_MPI_Alltoall_leave);
 
 	return res;
 }
@@ -313,16 +264,14 @@ int NAME_ROUTINE_C(MPI_Alltoallv) (MPI3_CONST void *sendbuf, MPI3_CONST int *sen
 {
 	int res;
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Alltoallv_enter (MPI3_VOID_P_CAST sendbuf, MPI3_VOID_P_CAST sendcounts, MPI3_VOID_P_CAST sdispls, sendtype, recvbuf, MPI3_VOID_P_CAST recvcounts, MPI3_VOID_P_CAST rdispls, recvtype, comm);
-#endif
+	DLB(DLB_MPI_Alltoallv_enter, MPI3_VOID_P_CAST sendbuf, MPI3_VOID_P_CAST sendcounts, MPI3_VOID_P_CAST sdispls, sendtype, recvbuf, MPI3_VOID_P_CAST recvcounts, MPI3_VOID_P_CAST rdispls, recvtype, comm);
 
 	Extrae_MPI_ProcessCollectiveCommunicator (comm);
 
 	if (mpitrace_on)
 	{
 		DEBUG_INTERFACE(ENTER)
-		Backend_Enter_Instrumentation (2+Caller_Count[CALLER_MPI]);
+		Backend_Enter_Instrumentation ();
 		res = MPI_Alltoallv_C_Wrapper
 		  (MPI3_VOID_P_CAST sendbuf, MPI3_VOID_P_CAST sendcounts, MPI3_VOID_P_CAST sdispls, sendtype, recvbuf, MPI3_VOID_P_CAST recvcounts,
 		  MPI3_VOID_P_CAST rdispls, recvtype, comm);
@@ -334,9 +283,7 @@ int NAME_ROUTINE_C(MPI_Alltoallv) (MPI3_CONST void *sendbuf, MPI3_CONST int *sen
 		  (sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts,
 		  rdispls, recvtype, comm);
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Alltoallv_leave ();
-#endif
+	DLB(DLB_MPI_Alltoallv_leave);
 
 	return res;
 }
@@ -350,16 +297,14 @@ int NAME_ROUTINE_C(MPI_Allgather) (MPI3_CONST void *sendbuf, int sendcount,
 {
 	int res;
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Allgather_enter (MPI3_VOID_P_CAST sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
-#endif
+	DLB(DLB_MPI_Allgather_enter, MPI3_VOID_P_CAST sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
 
 	Extrae_MPI_ProcessCollectiveCommunicator (comm);
 
 	if (mpitrace_on)
 	{
 		DEBUG_INTERFACE(ENTER)
-		Backend_Enter_Instrumentation (2+Caller_Count[CALLER_MPI]);
+		Backend_Enter_Instrumentation ();
 		res = MPI_Allgather_C_Wrapper
 		  (MPI3_VOID_P_CAST sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype,
 		  comm);
@@ -371,9 +316,7 @@ int NAME_ROUTINE_C(MPI_Allgather) (MPI3_CONST void *sendbuf, int sendcount,
 		  (sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype,
 		  comm);
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Allgather_leave ();
-#endif
+	DLB(DLB_MPI_Allgather_leave);
 
 	return res;
 }
@@ -387,16 +330,14 @@ int NAME_ROUTINE_C(MPI_Allgatherv) (MPI3_CONST void *sendbuf, int sendcount,
 {
 	int res;
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Allgatherv_enter (MPI3_VOID_P_CAST sendbuf, sendcount, sendtype, recvbuf, MPI3_C_INT_P_CAST recvcounts,MPI3_C_INT_P_CAST  displs, recvtype, comm);
-#endif
+	DLB(DLB_MPI_Allgatherv_enter, MPI3_VOID_P_CAST sendbuf, sendcount, sendtype, recvbuf, MPI3_C_INT_P_CAST recvcounts,MPI3_C_INT_P_CAST  displs, recvtype, comm);
 
 	Extrae_MPI_ProcessCollectiveCommunicator (comm);
 
 	if (mpitrace_on)
 	{
 		DEBUG_INTERFACE(ENTER)
-		Backend_Enter_Instrumentation (2+Caller_Count[CALLER_MPI]);
+		Backend_Enter_Instrumentation ();
 		res =  MPI_Allgatherv_C_Wrapper
 		  (MPI3_VOID_P_CAST sendbuf, sendcount, sendtype, recvbuf, MPI3_C_INT_P_CAST recvcounts, MPI3_C_INT_P_CAST displs,
 		  recvtype, comm);
@@ -408,9 +349,7 @@ int NAME_ROUTINE_C(MPI_Allgatherv) (MPI3_CONST void *sendbuf, int sendcount,
 		  (sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs,
 		  recvtype, comm);
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Allgatherv_leave ();
-#endif
+	DLB(DLB_MPI_Allgatherv_leave);
 
 	return res;
 }
@@ -424,16 +363,14 @@ int NAME_ROUTINE_C(MPI_Gather) (MPI3_CONST void *sendbuf, int sendcount,
 {
 	int res;
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Gather_enter (MPI3_VOID_P_CAST sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm);
-#endif
+	DLB(DLB_MPI_Gather_enter, MPI3_VOID_P_CAST sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm);
 
 	Extrae_MPI_ProcessCollectiveCommunicator (comm);
 
 	if (mpitrace_on)
 	{
 		DEBUG_INTERFACE(ENTER)
-		Backend_Enter_Instrumentation (2+Caller_Count[CALLER_MPI]);
+		Backend_Enter_Instrumentation ();
 		res =  MPI_Gather_C_Wrapper
 		  (MPI3_VOID_P_CAST sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root,
 		  comm);
@@ -445,9 +382,7 @@ int NAME_ROUTINE_C(MPI_Gather) (MPI3_CONST void *sendbuf, int sendcount,
 		  (sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root,
 		  comm);
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Gather_leave ();
-#endif
+	DLB(DLB_MPI_Gather_leave);
 
 	return res;
 }
@@ -461,16 +396,14 @@ int NAME_ROUTINE_C(MPI_Gatherv) (MPI3_CONST void *sendbuf, int sendcount,
 {
 	int res;
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Gatherv_enter (MPI3_VOID_P_CAST sendbuf, sendcount, sendtype, recvbuf, recvcounts, MPI3_C_INT_P_CAST displs, recvtype, root, comm);
-#endif
+	DLB(DLB_MPI_Gatherv_enter, MPI3_VOID_P_CAST sendbuf, sendcount, sendtype, recvbuf, recvcounts, MPI3_C_INT_P_CAST displs, recvtype, root, comm);
 
 	Extrae_MPI_ProcessCollectiveCommunicator (comm);
 
 	if (mpitrace_on)
 	{
 		DEBUG_INTERFACE(ENTER)
-		Backend_Enter_Instrumentation (2+Caller_Count[CALLER_MPI]);
+		Backend_Enter_Instrumentation ();
 		res =  MPI_Gatherv_C_Wrapper
             (MPI3_VOID_P_CAST sendbuf, sendcount, sendtype, recvbuf, MPI3_C_INT_P_CAST recvcounts, MPI3_C_INT_P_CAST displs,
              recvtype, root, comm);
@@ -482,9 +415,7 @@ int NAME_ROUTINE_C(MPI_Gatherv) (MPI3_CONST void *sendbuf, int sendcount,
 		  (sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs,
 		  recvtype, root, comm);
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Gatherv_leave ();
-#endif
+	DLB(DLB_MPI_Gatherv_leave);
 
 	return res;
 }
@@ -498,16 +429,14 @@ int NAME_ROUTINE_C(MPI_Scatter) (MPI3_CONST void *sendbuf, int sendcount,
 {
 	int res;
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Scatter_enter (MPI3_VOID_P_CAST sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm);
-#endif
+	DLB(DLB_MPI_Scatter_enter, MPI3_VOID_P_CAST sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm);
 
 	Extrae_MPI_ProcessCollectiveCommunicator (comm);
 
 	if (mpitrace_on)
 	{
 		DEBUG_INTERFACE(ENTER)
-		Backend_Enter_Instrumentation (2+Caller_Count[CALLER_MPI]);
+		Backend_Enter_Instrumentation ();
 		res =  MPI_Scatter_C_Wrapper
 		  (MPI3_VOID_P_CAST sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root,
 		  comm);
@@ -519,9 +448,7 @@ int NAME_ROUTINE_C(MPI_Scatter) (MPI3_CONST void *sendbuf, int sendcount,
 		  (sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root,
 		  comm);
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Scatter_leave ();
-#endif
+	DLB(DLB_MPI_Scatter_leave);
 
 	return res;
 }
@@ -535,16 +462,14 @@ int NAME_ROUTINE_C(MPI_Scatterv) (MPI3_CONST void *sendbuf, MPI3_CONST int *send
 {
 	int res;
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Scatterv_enter (MPI3_VOID_P_CAST sendbuf, MPI3_C_INT_P_CAST sendcounts, MPI3_C_INT_P_CAST displs, sendtype, recvbuf, recvcount, recvtype, root, comm);
-#endif
+	DLB(DLB_MPI_Scatterv_enter, MPI3_VOID_P_CAST sendbuf, MPI3_C_INT_P_CAST sendcounts, MPI3_C_INT_P_CAST displs, sendtype, recvbuf, recvcount, recvtype, root, comm);
 
 	Extrae_MPI_ProcessCollectiveCommunicator (comm);
 
 	if (mpitrace_on)
 	{
 		DEBUG_INTERFACE(ENTER)
-		Backend_Enter_Instrumentation (2+Caller_Count[CALLER_MPI]);
+		Backend_Enter_Instrumentation ();
 		res = MPI_Scatterv_C_Wrapper
             (MPI3_VOID_P_CAST sendbuf, MPI3_C_INT_P_CAST sendcounts, MPI3_C_INT_P_CAST displs, sendtype, recvbuf, recvcount,
              recvtype, root, comm);
@@ -556,9 +481,7 @@ int NAME_ROUTINE_C(MPI_Scatterv) (MPI3_CONST void *sendbuf, MPI3_CONST int *send
 		  (sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount,
 		  recvtype, root, comm);
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Scatterv_leave ();
-#endif
+	DLB(DLB_MPI_Scatterv_leave);
 
 	return res;
 }
@@ -571,16 +494,14 @@ int NAME_ROUTINE_C(MPI_Scan) (MPI3_CONST void *sendbuf, void *recvbuf, int count
 {
 	int res;
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Scan_enter (MPI3_VOID_P_CAST sendbuf, recvbuf, count, datatype, op, comm);
-#endif
+	DLB(DLB_MPI_Scan_enter, MPI3_VOID_P_CAST sendbuf, recvbuf, count, datatype, op, comm);
 
 	Extrae_MPI_ProcessCollectiveCommunicator (comm);
 
 	if (mpitrace_on)
 	{
 		DEBUG_INTERFACE(ENTER)
-		Backend_Enter_Instrumentation (2+Caller_Count[CALLER_MPI]);
+		Backend_Enter_Instrumentation ();
 		res = MPI_Scan_C_Wrapper (MPI3_VOID_P_CAST sendbuf, recvbuf, count, datatype, op, comm);
 		Backend_Leave_Instrumentation ();
 		DEBUG_INTERFACE(LEAVE)
@@ -588,9 +509,7 @@ int NAME_ROUTINE_C(MPI_Scan) (MPI3_CONST void *sendbuf, void *recvbuf, int count
 	else
 		res = PMPI_Scan (sendbuf, recvbuf, count, datatype, op, comm);
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Scan_leave ();
-#endif
+	DLB(DLB_MPI_Scan_leave);
 
 	return res;
 }
@@ -604,16 +523,14 @@ int NAME_ROUTINE_C(MPI_Ireduce) (MPI3_CONST void *sendbuf, void *recvbuf, int co
 {
 	int res;
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Ireduce_enter (MPI3_VOID_P_CAST sendbuf, recvbuf, count, datatype, op, root, comm, req);
-#endif
+	DLB(DLB_MPI_Ireduce_enter, MPI3_VOID_P_CAST sendbuf, recvbuf, count, datatype, op, root, comm, req);
 
 	Extrae_MPI_ProcessCollectiveCommunicator (comm);
 
 	if (mpitrace_on)
 	{
 		DEBUG_INTERFACE(ENTER)
-		Backend_Enter_Instrumentation (2+Caller_Count[CALLER_MPI]);
+		Backend_Enter_Instrumentation ();
 		res = MPI_Ireduce_C_Wrapper 
 			(MPI3_VOID_P_CAST sendbuf, recvbuf, count, datatype, op, root, comm, req);
 		Backend_Leave_Instrumentation ();
@@ -622,9 +539,7 @@ int NAME_ROUTINE_C(MPI_Ireduce) (MPI3_CONST void *sendbuf, void *recvbuf, int co
 	else
 		res = PMPI_Ireduce (sendbuf, recvbuf, count, datatype, op, root, comm, req);
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Ireduce_leave ();
-#endif
+	DLB(DLB_MPI_Ireduce_leave);
 
 	return res;
 }
@@ -638,16 +553,14 @@ int NAME_ROUTINE_C(MPI_Ireduce_scatter) (MPI3_CONST void *sendbuf, void *recvbuf
 {
 	int res;
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Ireduce_scatter_enter (MPI3_VOID_P_CAST sendbuf, recvbuf, MPI3_C_INT_P_CAST recvcounts, datatype, op, comm, req);
-#endif
+	DLB(DLB_MPI_Ireduce_scatter_enter, MPI3_VOID_P_CAST sendbuf, recvbuf, MPI3_C_INT_P_CAST recvcounts, datatype, op, comm, req);
 
 	Extrae_MPI_ProcessCollectiveCommunicator (comm);
 
 	if (mpitrace_on)
 	{
 		DEBUG_INTERFACE(ENTER)
-		Backend_Enter_Instrumentation (2+Caller_Count[CALLER_MPI]);
+		Backend_Enter_Instrumentation ();
 		res = MPI_Ireduce_Scatter_C_Wrapper (MPI3_VOID_P_CAST sendbuf, recvbuf, MPI3_C_INT_P_CAST recvcounts, datatype, op, comm, req);
 		Backend_Leave_Instrumentation ();
 		DEBUG_INTERFACE(LEAVE)
@@ -656,9 +569,7 @@ int NAME_ROUTINE_C(MPI_Ireduce_scatter) (MPI3_CONST void *sendbuf, void *recvbuf
 		res = PMPI_Ireduce_scatter (sendbuf, recvbuf, recvcounts, datatype, op,
 			comm, req);
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Ireduce_scatter_leave ();
-#endif
+	DLB(DLB_MPI_Ireduce_scatter_leave);
 
 	return res;
 }
@@ -671,16 +582,14 @@ int NAME_ROUTINE_C(MPI_Iallreduce) (MPI3_CONST void *sendbuf, void *recvbuf, int
 {
 	int res;
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Iallreduce_enter (MPI3_VOID_P_CAST sendbuf, recvbuf, count, datatype, op, comm, req);
-#endif
+	DLB(DLB_MPI_Iallreduce_enter, MPI3_VOID_P_CAST sendbuf, recvbuf, count, datatype, op, comm, req);
 
 	Extrae_MPI_ProcessCollectiveCommunicator (comm);
 
 	if (mpitrace_on)
 	{
 		DEBUG_INTERFACE(ENTER)
-		Backend_Enter_Instrumentation (2+Caller_Count[CALLER_MPI]);
+		Backend_Enter_Instrumentation ();
 		res = MPI_Iallreduce_C_Wrapper (MPI3_VOID_P_CAST sendbuf, recvbuf, count, datatype, op, comm, req);
 		Backend_Leave_Instrumentation ();
 		DEBUG_INTERFACE(LEAVE)
@@ -688,9 +597,7 @@ int NAME_ROUTINE_C(MPI_Iallreduce) (MPI3_CONST void *sendbuf, void *recvbuf, int
 	else
 		res = PMPI_Iallreduce (sendbuf, recvbuf, count, datatype, op, comm, req);
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Iallreduce_leave ();
-#endif
+	DLB(DLB_MPI_Iallreduce_leave);
 
 	return res;
 }
@@ -702,16 +609,14 @@ int NAME_ROUTINE_C(MPI_Ibarrier) (MPI_Comm comm, MPI_Request *req)
 {
 	int res;
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Ibarrier_enter (comm, req);
-#endif
+	DLB(DLB_MPI_Ibarrier_enter, comm, req);
 
 	Extrae_MPI_ProcessCollectiveCommunicator (comm);
 
 	if (mpitrace_on)
 	{
 		DEBUG_INTERFACE(ENTER)
-		Backend_Enter_Instrumentation (2+Caller_Count[CALLER_MPI]);
+		Backend_Enter_Instrumentation ();
 		res =  MPI_Ibarrier_C_Wrapper (comm, req);
 		Backend_Leave_Instrumentation ();
 		DEBUG_INTERFACE(LEAVE)
@@ -719,9 +624,7 @@ int NAME_ROUTINE_C(MPI_Ibarrier) (MPI_Comm comm, MPI_Request *req)
 	else
 		res = PMPI_Ibarrier (comm, req);
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Ibarrier_leave ();
-#endif
+	DLB(DLB_MPI_Ibarrier_leave);
 
 	return res;
 }
@@ -734,16 +637,14 @@ int NAME_ROUTINE_C(MPI_Ibcast) (void *buffer, int count, MPI_Datatype datatype,
 {
 	int res;
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Ibcast_enter (buffer, count, datatype, root, comm, req);
-#endif
+	DLB(DLB_MPI_Ibcast_enter, buffer, count, datatype, root, comm, req);
 
 	Extrae_MPI_ProcessCollectiveCommunicator (comm);
 
 	if (mpitrace_on)
 	{
 		DEBUG_INTERFACE(ENTER)
-		Backend_Enter_Instrumentation (2+Caller_Count[CALLER_MPI]);
+		Backend_Enter_Instrumentation ();
 		res = MPI_Ibcast_C_Wrapper (buffer, count, datatype, root, comm, req);
 		Backend_Leave_Instrumentation ();
 		DEBUG_INTERFACE(LEAVE)
@@ -751,15 +652,13 @@ int NAME_ROUTINE_C(MPI_Ibcast) (void *buffer, int count, MPI_Datatype datatype,
 	else
 		res = PMPI_Ibcast (buffer, count, datatype, root, comm, req);
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Ibcast_leave ();
-#endif
+	DLB(DLB_MPI_Ibcast_leave);
 
 	return res;
 }
 
 /******************************************************************************
- ***  MPI_Alltoall
+ ***  MPI_Ialltoall
  ******************************************************************************/
 int NAME_ROUTINE_C(MPI_Ialltoall) (MPI3_CONST void *sendbuf, int sendcount,
 	MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype,
@@ -767,16 +666,14 @@ int NAME_ROUTINE_C(MPI_Ialltoall) (MPI3_CONST void *sendbuf, int sendcount,
 {
 	int res;
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Ialltoall_enter (MPI3_VOID_P_CAST sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, req);
-#endif
+	DLB(DLB_MPI_Ialltoall_enter, MPI3_VOID_P_CAST sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, req);
 
 	Extrae_MPI_ProcessCollectiveCommunicator (comm);
 
 	if (mpitrace_on)
 	{
 		DEBUG_INTERFACE(ENTER)
-		Backend_Enter_Instrumentation (2+Caller_Count[CALLER_MPI]);
+		Backend_Enter_Instrumentation ();
 		res = MPI_Ialltoall_C_Wrapper
 		  (MPI3_VOID_P_CAST sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, req);
 		Backend_Leave_Instrumentation ();
@@ -786,9 +683,7 @@ int NAME_ROUTINE_C(MPI_Ialltoall) (MPI3_CONST void *sendbuf, int sendcount,
 		res = PMPI_Ialltoall
 		  (sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, req);
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Ialltoall_leave ();
-#endif
+	DLB(DLB_MPI_Ialltoall_leave);
 
 	return res;
 }
@@ -802,16 +697,14 @@ int NAME_ROUTINE_C(MPI_Ialltoallv) (MPI3_CONST void *sendbuf, MPI3_CONST int *se
 {
 	int res;
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Ialltoallv_enter (MPI3_VOID_P_CAST sendbuf, MPI3_VOID_P_CAST sendcounts, MPI3_VOID_P_CAST sdispls, sendtype, recvbuf, MPI3_VOID_P_CAST recvcounts, MPI3_VOID_P_CAST rdispls, recvtype, comm, req);
-#endif
+	DLB(DLB_MPI_Ialltoallv_enter, MPI3_VOID_P_CAST sendbuf, MPI3_VOID_P_CAST sendcounts, MPI3_VOID_P_CAST sdispls, sendtype, recvbuf, MPI3_VOID_P_CAST recvcounts, MPI3_VOID_P_CAST rdispls, recvtype, comm, req);
 
 	Extrae_MPI_ProcessCollectiveCommunicator (comm);
 
 	if (mpitrace_on)
 	{
 		DEBUG_INTERFACE(ENTER)
-		Backend_Enter_Instrumentation (2+Caller_Count[CALLER_MPI]);
+		Backend_Enter_Instrumentation ();
 		res = MPI_Ialltoallv_C_Wrapper
 		  (MPI3_VOID_P_CAST sendbuf, MPI3_VOID_P_CAST sendcounts, MPI3_VOID_P_CAST sdispls, sendtype, recvbuf, MPI3_VOID_P_CAST recvcounts,
 		  MPI3_VOID_P_CAST rdispls, recvtype, comm, req);
@@ -823,9 +716,7 @@ int NAME_ROUTINE_C(MPI_Ialltoallv) (MPI3_CONST void *sendbuf, MPI3_CONST int *se
 		  (sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts,
 		  rdispls, recvtype, comm, req);
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Alltoallv_leave ();
-#endif
+	DLB(DLB_MPI_Alltoallv_leave);
 
 	return res;
 }
@@ -839,16 +730,14 @@ int NAME_ROUTINE_C(MPI_Iallgather) (MPI3_CONST void *sendbuf, int sendcount,
 {
 	int res;
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Iallgather_enter (MPI3_VOID_P_CAST sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, req);
-#endif
+	DLB(DLB_MPI_Iallgather_enter, MPI3_VOID_P_CAST sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, req);
 
 	Extrae_MPI_ProcessCollectiveCommunicator (comm);
 
 	if (mpitrace_on)
 	{
 		DEBUG_INTERFACE(ENTER)
-		Backend_Enter_Instrumentation (2+Caller_Count[CALLER_MPI]);
+		Backend_Enter_Instrumentation ();
 		res = MPI_Iallgather_C_Wrapper
 		  (MPI3_VOID_P_CAST sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype,
 		  comm, req);
@@ -860,9 +749,7 @@ int NAME_ROUTINE_C(MPI_Iallgather) (MPI3_CONST void *sendbuf, int sendcount,
 		  (sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype,
 		  comm, req);
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Iallgather_leave ();
-#endif
+	DLB(DLB_MPI_Iallgather_leave);
 
 	return res;
 }
@@ -876,16 +763,14 @@ int NAME_ROUTINE_C(MPI_Iallgatherv) (MPI3_CONST void *sendbuf, int sendcount,
 {
 	int res;
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Iallgatherv_enter (MPI3_VOID_P_CAST sendbuf, sendcount, sendtype, recvbuf, MPI3_C_INT_P_CAST recvcounts,MPI3_C_INT_P_CAST  displs, recvtype, comm, req);
-#endif
+	DLB(DLB_MPI_Iallgatherv_enter, MPI3_VOID_P_CAST sendbuf, sendcount, sendtype, recvbuf, MPI3_C_INT_P_CAST recvcounts,MPI3_C_INT_P_CAST  displs, recvtype, comm, req);
 
 	Extrae_MPI_ProcessCollectiveCommunicator (comm);
 
 	if (mpitrace_on)
 	{
 		DEBUG_INTERFACE(ENTER)
-		Backend_Enter_Instrumentation (2+Caller_Count[CALLER_MPI]);
+		Backend_Enter_Instrumentation ();
 		res =  MPI_Iallgatherv_C_Wrapper
 		  (MPI3_VOID_P_CAST sendbuf, sendcount, sendtype, recvbuf, MPI3_C_INT_P_CAST recvcounts, MPI3_C_INT_P_CAST displs,
 		  recvtype, comm, req);
@@ -897,9 +782,7 @@ int NAME_ROUTINE_C(MPI_Iallgatherv) (MPI3_CONST void *sendbuf, int sendcount,
 		  (sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs,
 		  recvtype, comm, req);
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Iallgatherv_leave ();
-#endif
+	DLB(DLB_MPI_Iallgatherv_leave);
 
 	return res;
 }
@@ -913,16 +796,14 @@ int NAME_ROUTINE_C(MPI_Igather) (MPI3_CONST void *sendbuf, int sendcount,
 {
 	int res;
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Igather_enter (MPI3_VOID_P_CAST sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm, req);
-#endif
+	DLB(DLB_MPI_Igather_enter, MPI3_VOID_P_CAST sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm, req);
 
 	Extrae_MPI_ProcessCollectiveCommunicator (comm);
 
 	if (mpitrace_on)
 	{
 		DEBUG_INTERFACE(ENTER)
-		Backend_Enter_Instrumentation (2+Caller_Count[CALLER_MPI]);
+		Backend_Enter_Instrumentation ();
 		res =  MPI_Igather_C_Wrapper
 		  (MPI3_VOID_P_CAST sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root,
 		  comm, req);
@@ -934,9 +815,7 @@ int NAME_ROUTINE_C(MPI_Igather) (MPI3_CONST void *sendbuf, int sendcount,
 		  (sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root,
 		  comm, req);
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Igather_leave ();
-#endif
+	DLB(DLB_MPI_Igather_leave);
 
 	return res;
 }
@@ -950,16 +829,14 @@ int NAME_ROUTINE_C(MPI_Igatherv) (MPI3_CONST void *sendbuf, int sendcount,
 {
 	int res;
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Igatherv_enter (MPI3_VOID_P_CAST sendbuf, sendcount, sendtype, recvbuf, recvcounts, MPI3_C_INT_P_CAST displs, recvtype, root, comm, req);
-#endif
+	DLB(DLB_MPI_Igatherv_enter, MPI3_VOID_P_CAST sendbuf, sendcount, sendtype, recvbuf, recvcounts, MPI3_C_INT_P_CAST displs, recvtype, root, comm, req);
 
 	Extrae_MPI_ProcessCollectiveCommunicator (comm);
 
 	if (mpitrace_on)
 	{
 		DEBUG_INTERFACE(ENTER)
-		Backend_Enter_Instrumentation (2+Caller_Count[CALLER_MPI]);
+		Backend_Enter_Instrumentation ();
 		res =  MPI_Igatherv_C_Wrapper
             (MPI3_VOID_P_CAST sendbuf, sendcount, sendtype, recvbuf, MPI3_C_INT_P_CAST recvcounts, MPI3_C_INT_P_CAST displs,
              recvtype, root, comm, req);
@@ -971,15 +848,13 @@ int NAME_ROUTINE_C(MPI_Igatherv) (MPI3_CONST void *sendbuf, int sendcount,
 		  (sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs,
 		  recvtype, root, comm, req);
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Igatherv_leave ();
-#endif
+	DLB(DLB_MPI_Igatherv_leave);
 
 	return res;
 }
 
 /******************************************************************************
- ***  MPI_Scatter
+ ***  MPI_Iscatter
  ******************************************************************************/
 int NAME_ROUTINE_C(MPI_Iscatter) (MPI3_CONST void *sendbuf, int sendcount,
 	MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype,
@@ -987,16 +862,14 @@ int NAME_ROUTINE_C(MPI_Iscatter) (MPI3_CONST void *sendbuf, int sendcount,
 {
 	int res;
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Iscatter_enter (MPI3_VOID_P_CAST sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm, req);
-#endif
+	DLB(DLB_MPI_Iscatter_enter, MPI3_VOID_P_CAST sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm, req);
 
 	Extrae_MPI_ProcessCollectiveCommunicator (comm);
 
 	if (mpitrace_on)
 	{
 		DEBUG_INTERFACE(ENTER)
-		Backend_Enter_Instrumentation (2+Caller_Count[CALLER_MPI]);
+		Backend_Enter_Instrumentation ();
 		res =  MPI_Iscatter_C_Wrapper
 		  (MPI3_VOID_P_CAST sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root,
 		  comm, req);
@@ -1008,15 +881,13 @@ int NAME_ROUTINE_C(MPI_Iscatter) (MPI3_CONST void *sendbuf, int sendcount,
 		  (sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root,
 		  comm, req);
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Iscatter_leave ();
-#endif
+	DLB(DLB_MPI_Iscatter_leave);
 
 	return res;
 }
 
 /******************************************************************************
- ***  MPI_IScatterv
+ ***  MPI_Iscatterv
  ******************************************************************************/
 int NAME_ROUTINE_C(MPI_Iscatterv) (MPI3_CONST void *sendbuf, MPI3_CONST int *sendcounts, MPI3_CONST int *displs, 
 	MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype,
@@ -1024,16 +895,14 @@ int NAME_ROUTINE_C(MPI_Iscatterv) (MPI3_CONST void *sendbuf, MPI3_CONST int *sen
 {
 	int res;
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Iscatterv_enter (MPI3_VOID_P_CAST sendbuf, MPI3_C_INT_P_CAST sendcounts, MPI3_C_INT_P_CAST displs, sendtype, recvbuf, recvcount, recvtype, root, comm, req);
-#endif
+	DLB(DLB_MPI_Iscatterv_enter, MPI3_VOID_P_CAST sendbuf, MPI3_C_INT_P_CAST sendcounts, MPI3_C_INT_P_CAST displs, sendtype, recvbuf, recvcount, recvtype, root, comm, req);
 
 	Extrae_MPI_ProcessCollectiveCommunicator (comm);
 
 	if (mpitrace_on)
 	{
 		DEBUG_INTERFACE(ENTER)
-		Backend_Enter_Instrumentation (2+Caller_Count[CALLER_MPI]);
+		Backend_Enter_Instrumentation ();
 		res = MPI_Iscatterv_C_Wrapper
             (MPI3_VOID_P_CAST sendbuf, MPI3_C_INT_P_CAST sendcounts, MPI3_C_INT_P_CAST displs, sendtype, recvbuf, recvcount, recvtype, root, comm, req);
 		Backend_Leave_Instrumentation ();
@@ -1044,31 +913,27 @@ int NAME_ROUTINE_C(MPI_Iscatterv) (MPI3_CONST void *sendbuf, MPI3_CONST int *sen
 		  (sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount,
 		  recvtype, root, comm, req);
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Iscatterv_leave ();
-#endif
+	DLB(DLB_MPI_Iscatterv_leave);
 
 	return res;
 }
 
 /******************************************************************************
- ***  MPI_Scan
+ ***  MPI_Iscan
  ******************************************************************************/
 int NAME_ROUTINE_C(MPI_Iscan) (MPI3_CONST void *sendbuf, void *recvbuf, int count,
 	MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *req)
 {
 	int res;
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Iscan_enter (MPI3_VOID_P_CAST sendbuf, recvbuf, count, datatype, op, comm, req);
-#endif
+	DLB(DLB_MPI_Iscan_enter, MPI3_VOID_P_CAST sendbuf, recvbuf, count, datatype, op, comm, req);
 
 	Extrae_MPI_ProcessCollectiveCommunicator (comm);
 
 	if (mpitrace_on)
 	{
 		DEBUG_INTERFACE(ENTER)
-		Backend_Enter_Instrumentation (2+Caller_Count[CALLER_MPI]);
+		Backend_Enter_Instrumentation ();
 		res = MPI_Iscan_C_Wrapper (MPI3_VOID_P_CAST sendbuf, recvbuf, count, datatype, op, comm, req);
 		Backend_Leave_Instrumentation ();
 		DEBUG_INTERFACE(LEAVE)
@@ -1076,9 +941,136 @@ int NAME_ROUTINE_C(MPI_Iscan) (MPI3_CONST void *sendbuf, void *recvbuf, int coun
 	else
 		res = PMPI_Iscan (sendbuf, recvbuf, count, datatype, op, comm, req);
 
-#if defined(ENABLE_LOAD_BALANCING)
-	DLB_MPI_Iscan_leave ();
-#endif
+	DLB(DLB_MPI_Iscan_leave);
+
+	return res;
+}
+
+/******************************************************************************
+ ***  MPI_Reduce_scatter_block
+ ******************************************************************************/
+int NAME_ROUTINE_C(MPI_Reduce_scatter_block) (MPI3_CONST void *sendbuf, void *recvbuf,
+	int recvcount, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm)
+{
+	int res;
+
+	DLB(DLB_MPI_Reduce_scatter_block_enter, MPI3_VOID_P_CAST sendbuf, recvbuf, recvcount, datatype, op, comm);
+
+	Extrae_MPI_ProcessCollectiveCommunicator (comm);
+
+	if (mpitrace_on)
+	{
+		DEBUG_INTERFACE(ENTER)
+		Backend_Enter_Instrumentation ();
+		res = MPI_Reduce_Scatter_Block_C_Wrapper (MPI3_VOID_P_CAST sendbuf, recvbuf, recvcount, datatype,
+			op, comm);
+		Backend_Leave_Instrumentation ();
+		DEBUG_INTERFACE(LEAVE)
+	}
+	else
+		res = PMPI_Reduce_scatter_block (sendbuf, recvbuf, recvcount, datatype, op,
+			comm);
+
+	DLB(DLB_MPI_Reduce_scatter_block_leave);
+
+	return res;
+}
+
+/******************************************************************************
+ ***  MPI_Ireduce_scatter_block
+ ******************************************************************************/
+int NAME_ROUTINE_C(MPI_Ireduce_scatter_block) (MPI3_CONST void *sendbuf, void *recvbuf,
+	int recvcount, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm,
+	MPI_Request *req)
+{
+	int res;
+
+	DLB(DLB_MPI_Ireduce_scatter_block_enter, MPI3_VOID_P_CAST sendbuf, recvbuf, recvcount, datatype, op, comm, req);
+
+	Extrae_MPI_ProcessCollectiveCommunicator (comm);
+
+	if (mpitrace_on)
+	{
+		DEBUG_INTERFACE(ENTER)
+		Backend_Enter_Instrumentation ();
+		res = MPI_Ireduce_Scatter_Block_C_Wrapper (MPI3_VOID_P_CAST sendbuf, recvbuf, recvcount, datatype, op, comm, req);
+		Backend_Leave_Instrumentation ();
+		DEBUG_INTERFACE(LEAVE)
+	}
+	else
+		res = PMPI_Ireduce_scatter_block (sendbuf, recvbuf, recvcount, datatype, op,
+			comm, req);
+
+	DLB(DLB_MPI_Ireduce_scatter_block_leave);
+
+	return res;
+}
+
+/******************************************************************************
+ ***  MPI_Alltoallw
+ ******************************************************************************/
+int NAME_ROUTINE_C(MPI_Alltoallw) (MPI3_CONST void *sendbuf, MPI3_CONST int *sendcounts, MPI3_CONST int *sdispls,
+	MPI3_CONST MPI_Datatype *sendtypes, void *recvbuf, MPI3_CONST int *recvcounts, MPI3_CONST int *rdispls,
+	MPI3_CONST MPI_Datatype *recvtypes, MPI_Comm comm)
+{
+	int res;
+
+	DLB(DLB_MPI_Alltoallw_enter, MPI3_VOID_P_CAST sendbuf, MPI3_C_INT_P_CAST sendcounts, MPI3_C_INT_P_CAST sdispls, MPI3_MPI_DATATYPE_P_CAST sendtypes, recvbuf, MPI3_C_INT_P_CAST recvcounts, MPI3_C_INT_P_CAST rdispls, MPI3_MPI_DATATYPE_P_CAST recvtypes, comm);
+
+	Extrae_MPI_ProcessCollectiveCommunicator (comm);
+
+	if (mpitrace_on)
+	{
+		DEBUG_INTERFACE(ENTER)
+		Backend_Enter_Instrumentation ();
+		res = MPI_Alltoallw_C_Wrapper
+		  (MPI3_VOID_P_CAST sendbuf, MPI3_C_INT_P_CAST sendcounts, MPI3_C_INT_P_CAST sdispls, MPI3_MPI_DATATYPE_P_CAST sendtypes, recvbuf, MPI3_C_INT_P_CAST recvcounts,
+		  MPI3_C_INT_P_CAST rdispls, MPI3_MPI_DATATYPE_P_CAST recvtypes, comm);
+		Backend_Leave_Instrumentation ();
+		DEBUG_INTERFACE(LEAVE)
+	}
+	else
+		res = PMPI_Alltoallw
+		  (sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts,
+		  rdispls, recvtypes, comm);
+
+	DLB(DLB_MPI_Alltoallw_leave);
+
+	return res;
+}
+
+/******************************************************************************
+ ***  MPI_Ialltoallw
+ ******************************************************************************/
+int NAME_ROUTINE_C(MPI_Ialltoallw) (MPI3_CONST void *sendbuf, MPI3_CONST int *sendcounts, MPI3_CONST int *sdispls,
+	MPI3_CONST MPI_Datatype *sendtypes, void *recvbuf, MPI3_CONST int *recvcounts, MPI3_CONST int *rdispls,
+	MPI3_CONST MPI_Datatype *recvtypes, MPI_Comm comm, MPI_Request *req)
+{
+	int res;
+
+	DLB(DLB_MPI_Ialltoallw_enter, MPI3_VOID_P_CAST sendbuf, MPI3_C_INT_P_CAST sendcounts, MPI3_C_INT_P_CAST sdispls, MPI3_MPI_DATATYPE_P_CAST sendtypes, recvbuf, MPI3_C_INT_P_CAST recvcounts, MPI3_C_INT_P_CAST rdispls, MPI3_MPI_DATATYPE_P_CAST recvtypes, comm, req);
+
+ 
+
+	Extrae_MPI_ProcessCollectiveCommunicator (comm);
+
+	if (mpitrace_on)
+	{
+		DEBUG_INTERFACE(ENTER)
+		Backend_Enter_Instrumentation ();
+		res = MPI_Ialltoallw_C_Wrapper
+		  (MPI3_VOID_P_CAST sendbuf, MPI3_C_INT_P_CAST sendcounts, MPI3_C_INT_P_CAST sdispls, MPI3_MPI_DATATYPE_P_CAST sendtypes, recvbuf, MPI3_C_INT_P_CAST recvcounts,
+		  MPI3_C_INT_P_CAST rdispls, MPI3_MPI_DATATYPE_P_CAST recvtypes, comm, req);
+		Backend_Leave_Instrumentation ();
+		DEBUG_INTERFACE(LEAVE)
+	}
+	else
+		res = PMPI_Ialltoallw
+		  (sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts,
+		  rdispls, recvtypes, comm, req);
+
+	DLB(DLB_MPI_Ialltoallw_leave);
+
 
 	return res;
 }
